@@ -5,22 +5,23 @@ JC cli module
 import io
 import sys
 import os
+import re
 from datetime import datetime, timezone
 import textwrap
-import signal
 import shlex
 import subprocess
-from typing import List, Union, Optional, TextIO
+from typing import List, Dict, Iterable, Union, Optional, TextIO
 from types import ModuleType
 from .lib import (
-    __version__, parser_info, all_parser_info, parsers, _get_parser, _parser_is_streaming,
-    parser_mod_list, standard_parser_mod_list, plugin_parser_mod_list, streaming_parser_mod_list
+    __version__, parser_info, all_parser_info, parsers, get_parser, _parser_is_streaming,
+    parser_mod_list, standard_parser_mod_list, plugin_parser_mod_list, streaming_parser_mod_list,
+    slurpable_parser_mod_list, _parser_is_slurpable
 )
-from .jc_types import JSONDictType, AboutJCType, MetadataType, CustomColorType, ParserInfoType
+from .jc_types import JSONDictType, CustomColorType, ParserInfoType
 from . import utils
 from .cli_data import (
     long_options_map, new_pygments_colors, old_pygments_colors, helptext_preamble_string,
-    helptext_end_string
+    slicetext_string, helptext_end_string
 )
 from .shell_completions import bash_completion, zsh_completion
 from . import tracebackplus
@@ -41,6 +42,8 @@ except Exception:
 JC_CLEAN_EXIT: int = 0
 JC_ERROR_EXIT: int = 100
 MAX_EXIT: int = 255
+SLICER_PATTERN: str = r'-?[0-9]*\:-?[0-9]*$'
+SLICER_RE = re.compile(SLICER_PATTERN)
 
 
 class info():
@@ -49,7 +52,7 @@ class info():
     author: str = 'Kelly Brazil'
     author_email: str = 'kellyjonbrazil@gmail.com'
     website: str = 'https://github.com/kellyjonbrazil/jc'
-    copyright: str = '© 2019-2022 Kelly Brazil'
+    copyright: str = '© 2019-2025 Kelly Brazil'
     license: str = 'MIT License'
 
 
@@ -63,19 +66,21 @@ if PYGMENTS_INSTALLED:
 
 
 class JcCli():
-    __slots__ = (
-        'data_in', 'data_out', 'options', 'args', 'parser_module', 'parser_name', 'indent', 'pad',
-        'custom_colors', 'show_hidden', 'ascii_only', 'json_separators', 'json_indent',
-        'run_timestamp', 'about', 'debug', 'verbose_debug', 'force_color', 'mono', 'help_me',
-        'pretty', 'quiet', 'ignore_exceptions', 'raw', 'meta_out', 'unbuffer', 'version_info',
-        'yaml_output', 'bash_comp', 'zsh_comp', 'magic_found_parser', 'magic_options',
-        'magic_run_command', 'magic_run_command_str', 'magic_stdout', 'magic_stderr',
-        'magic_returncode'
-    )
+    __slots__ = ('data_in', 'data_out', 'options', 'args', 'parser_module',
+                 'parser_name', 'indent', 'pad', 'custom_colors',
+                 'show_hidden', 'show_categories', 'ascii_only',
+                 'json_separators', 'json_indent', 'run_timestamp',
+                 'inputlist', 'about', 'debug', 'verbose_debug',
+                 'force_color', 'mono', 'help_me', 'pretty', 'quiet',
+                 'ignore_exceptions', 'raw', 'slurp', 'meta_out', 'unbuffer',
+                 'version_info', 'yaml_output', 'bash_comp', 'zsh_comp',
+                 'magic_found_parser', 'magic_options', 'magic_run_command',
+                 'magic_run_command_str', 'magic_stdout', 'magic_stderr',
+                 'magic_returncode', 'slice_str', 'slice_start', 'slice_end')
 
     def __init__(self) -> None:
-        self.data_in: Optional[Union[str, bytes, TextIO]] = None
-        self.data_out: Optional[Union[List[JSONDictType], JSONDictType, AboutJCType]] = None
+        self.data_in: Optional[Union[str, bytes, TextIO, Iterable[str]]] = None
+        self.data_out: Optional[Union[List[JSONDictType], JSONDictType]] = None
         self.options: List[str] = []
         self.args: List[str] = []
         self.parser_module: Optional[ModuleType] = None
@@ -84,10 +89,17 @@ class JcCli():
         self.pad: int = 0
         self.custom_colors: CustomColorType = {}
         self.show_hidden: bool = False
+        self.show_categories: bool = False
         self.ascii_only: bool = False
         self.json_separators: Optional[tuple[str, str]] = (',', ':')
         self.json_indent: Optional[int] = None
         self.run_timestamp: Optional[datetime] = None
+        self.inputlist: Optional[List[str]] = None
+
+        # slicer
+        self.slice_str: str = ''
+        self.slice_start: Optional[int] = None
+        self.slice_end: Optional[int] = None
 
         # cli options
         self.about: bool = False
@@ -100,6 +112,7 @@ class JcCli():
         self.quiet: bool = False
         self.ignore_exceptions: bool = False
         self.raw: bool = False
+        self.slurp: bool = False
         self.meta_out: bool = False
         self.unbuffer: bool = False
         self.version_info: bool = False
@@ -112,7 +125,7 @@ class JcCli():
         self.magic_options: List[str] = []
         self.magic_run_command: Optional[List[str]] = None
         self.magic_run_command_str: str = ''
-        self.magic_stdout: Optional[str] = None
+        self.magic_stdout: Optional[Union[str, Iterable[str]]] = None
         self.magic_stderr: Optional[str] = None
         self.magic_returncode: int = 0
 
@@ -134,33 +147,34 @@ class JcCli():
         JC_COLORS=blue,brightblack,magenta,green
         JC_COLORS=default,default,default,default
         """
-        input_error = False
-        env_colors = os.getenv('JC_COLORS')
+        if PYGMENTS_INSTALLED:
+            input_error = False
+            env_colors = os.getenv('JC_COLORS')
 
-        if env_colors:
-            color_list = env_colors.split(',')
-        else:
-            color_list = ['default', 'default', 'default', 'default']
+            if env_colors:
+                color_list = env_colors.split(',')
+            else:
+                color_list = ['default', 'default', 'default', 'default']
 
-        if len(color_list) != 4:
-            input_error = True
-
-        for color in color_list:
-            if color != 'default' and color not in PYGMENT_COLOR:
+            if len(color_list) != 4:
                 input_error = True
 
-        # if there is an issue with the env variable, just set all colors to default and move on
-        if input_error:
-            utils.warning_message(['Could not parse JC_COLORS environment variable'])
-            color_list = ['default', 'default', 'default', 'default']
+            for color in color_list:
+                if color != 'default' and color not in PYGMENT_COLOR:
+                    input_error = True
 
-        # Try the color set in the JC_COLORS env variable first. If it is set to default, then fall back to default colors
-        self.custom_colors = {
-            Name.Tag: f'bold {PYGMENT_COLOR[color_list[0]]}' if color_list[0] != 'default' else f"bold {PYGMENT_COLOR['blue']}",   # key names
-            Keyword: PYGMENT_COLOR[color_list[1]] if color_list[1] != 'default' else PYGMENT_COLOR['brightblack'],                 # true, false, null
-            Number: PYGMENT_COLOR[color_list[2]] if color_list[2] != 'default' else PYGMENT_COLOR['magenta'],                      # numbers
-            String: PYGMENT_COLOR[color_list[3]] if color_list[3] != 'default' else PYGMENT_COLOR['green']                         # strings
-        }
+            # if there is an issue with the env variable, just set all colors to default and move on
+            if input_error:
+                utils.warning_message(['Could not parse JC_COLORS environment variable'])
+                color_list = ['default', 'default', 'default', 'default']
+
+            # Try the color set in the JC_COLORS env variable first. If it is set to default, then fall back to default colors
+            self.custom_colors = {
+                Name.Tag: f'bold {PYGMENT_COLOR[color_list[0]]}' if color_list[0] != 'default' else f"bold {PYGMENT_COLOR['blue']}",   # key names
+                Keyword: PYGMENT_COLOR[color_list[1]] if color_list[1] != 'default' else PYGMENT_COLOR['brightblack'],                 # true, false, null
+                Number: PYGMENT_COLOR[color_list[2]] if color_list[2] != 'default' else PYGMENT_COLOR['magenta'],                      # numbers
+                String: PYGMENT_COLOR[color_list[3]] if color_list[3] != 'default' else PYGMENT_COLOR['green']                         # strings
+            }
 
     def set_mono(self) -> None:
         """
@@ -183,7 +197,8 @@ class JcCli():
     @staticmethod
     def parser_shortname(parser_arg: str) -> str:
         """Return short name of the parser with dashes and no -- prefix"""
-        return parser_arg[2:]
+        p = parser_arg.lstrip('-')
+        return p.replace('_', '-')
 
     def parsers_text(self) -> str:
         """Return the argument and description information from each parser"""
@@ -199,6 +214,43 @@ class JcCli():
 
         return ptext
 
+    def parser_categories_text(self) -> str:
+        """Return lists of parsers by category"""
+        category_text: str = ''
+        padding_char: str = ' '
+        all_parsers = all_parser_info(show_hidden=True, show_deprecated=False)
+        generic = [{'arg': x['argument'], 'desc': x['description']} for x in all_parsers if 'generic' in x.get('tags', [])]
+        standard = [{'arg': x['argument'], 'desc': x['description']} for x in all_parsers if 'standard' in x.get('tags', [])]
+        command = [{'arg': x['argument'], 'desc': x['description']} for x in all_parsers if 'command' in x.get('tags', [])]
+        slurpable = [{'arg': x['argument'], 'desc': x['description']} for x in all_parsers if 'slurpable' in x.get('tags', [])]
+        file_str_bin = [
+            {'arg': x['argument'], 'desc': x['description']} for x in all_parsers
+                if 'file' in x.get('tags', []) or
+                'string' in x.get('tags', []) or
+                'binary' in x.get('tags', [])
+        ]
+        streaming = [{'arg': x['argument'], 'desc': x['description']} for x in all_parsers if x.get('streaming')]
+        categories: Dict = {
+            'Generic Parsers:': generic,
+            'Standard Spec Parsers:': standard,
+            'File/String/Binary Parsers:': file_str_bin,
+            'Slurpable Parsers:': slurpable,
+            'Streaming Parsers:': streaming,
+            'Command Parsers:': command
+        }
+
+        for cat, cat_objs in categories.items():
+            category_text += f'{cat}  ({len(cat_objs)})\n'
+            for p in cat_objs:
+                parser_arg: str = p.get('arg', 'UNKNOWN')
+                parser_desc: str = p.get('desc', 'No description available.')
+                padding: int = self.pad - len(parser_arg)
+                padding_text: str = padding_char * padding
+                category_text += f'{parser_arg}{padding_text}{parser_desc}\n'
+            category_text += '\n'
+
+        return category_text[:-1]
+
     def options_text(self) -> str:
         """Return the argument and description information from each option"""
         otext: str = ''
@@ -215,7 +267,7 @@ class JcCli():
         return otext
 
     @staticmethod
-    def about_jc() -> AboutJCType:
+    def about_jc() -> JSONDictType:
         """Return jc info and the contents of each parser.info as a dictionary"""
         return {
             'name': 'jc',
@@ -232,16 +284,15 @@ class JcCli():
             'standard_parser_count': len(standard_parser_mod_list(show_hidden=True, show_deprecated=True)),
             'streaming_parser_count': len(streaming_parser_mod_list(show_hidden=True, show_deprecated=True)),
             'plugin_parser_count': len(plugin_parser_mod_list(show_hidden=True, show_deprecated=True)),
+            'slurpable_parser_count': len(slurpable_parser_mod_list(show_hidden=True, show_deprecated=True)),
             'parsers': all_parser_info(show_hidden=True, show_deprecated=True)
         }
 
     def helptext(self) -> str:
         """Return the help text with the list of parsers"""
-        self.indent = 4
-        self.pad = 20
         parsers_string: str = self.parsers_text()
         options_string: str = self.options_text()
-        helptext_string: str = f'{helptext_preamble_string}{parsers_string}\nOptions:\n{options_string}\n{helptext_end_string}'
+        helptext_string: str = f'{helptext_preamble_string}{parsers_string}\nOptions:\n{options_string}\n{slicetext_string}\n{helptext_end_string}'
         return helptext_string
 
     def help_doc(self) -> None:
@@ -249,6 +300,13 @@ class JcCli():
         Pages the parser documentation if a parser is found in the arguments,
         otherwise the general help text is printed.
         """
+        self.indent = 4
+        self.pad = 22
+
+        if self.show_categories:
+            utils._safe_print(self.parser_categories_text())
+            return
+
         for arg in self.args:
             parser_name: str = self.parser_shortname(arg)
 
@@ -259,9 +317,15 @@ class JcCli():
                 version: str = p_info.get('version', 'unknown')
                 author: str = p_info.get('author', 'unknown')
                 author_email: str = p_info.get('author_email', 'unknown')
+
+                slurpy = ''
+                if 'slurpable' in p_info.get('tags', []):
+                    slurpy = 'This parser can be used with the `--slurp` command-line option.\n\n'
+
                 doc_text: str = \
-                    f'{docs}\n'\
-                    f'Compatibility:  {compatible}\n\n'\
+                    f'{docs}\n' \
+                    f'Compatibility:  {compatible}\n\n' \
+                    f'{slurpy}' \
                     f'Version {version} by {author} ({author_email})\n'
 
                 utils._safe_pager(doc_text)
@@ -337,14 +401,19 @@ class JcCli():
             self.json_indent = 2
             self.json_separators = None
 
+        # Convert any non-serializable object to a string
+        def string_serializer(data):
+            return str(data)
+
         j_string = json.dumps(
             self.data_out,
             indent=self.json_indent,
             separators=self.json_separators,
-            ensure_ascii=self.ascii_only
+            ensure_ascii=self.ascii_only,
+            default=string_serializer
         )
 
-        if not self.mono:
+        if not self.mono and PYGMENTS_INSTALLED:
             class JcStyle(Style):
                 styles: CustomColorType = self.custom_colors
 
@@ -392,6 +461,17 @@ class JcCli():
                 self.magic_options = []
                 return
 
+            # slicer found
+            if ':' in arg:
+                if SLICER_RE.match(arg):
+                    self.slice_str = arg
+                    args_given.pop(0)
+                    continue
+                else:
+                    utils.warning_message(['Invalid slice syntax.'])
+                    args_given.pop(0)
+                    continue
+
             # option found - populate option list
             if arg.startswith('-'):
                 self.magic_options.extend(args_given.pop(0)[1:])
@@ -457,11 +537,33 @@ class JcCli():
 
         Supports running magic commands or opening /proc files to set the
         output to magic_stdout.
+
+        If multiple /proc files are detected, then a list of string output
+        is sent to self.magic_stdout and a corresponding list of proc filenames
+        is sent to self.inputlist.
         """
         if self.magic_run_command_str.startswith('/proc'):
             try:
                 self.magic_found_parser = 'proc'
-                self.magic_stdout = self.open_text_file(self.magic_run_command_str)
+                filelist = shlex.split(self.magic_run_command_str)
+
+                # multiple proc files detected
+                if len(filelist) > 1:
+                    self.slurp = True
+                    multi_out: List[str] = []
+                    self.inputlist = filelist
+
+                    for file in self.inputlist:
+                        # multi_out.append(self.open_text_file('/Users/kelly/temp' + file))
+                        multi_out.append(self.open_text_file(file))
+
+                    self.magic_stdout = multi_out
+
+                # single proc file
+                else:
+                    file = filelist[0]
+                    # self.magic_stdout = self.open_text_file('/Users/kelly/temp' + file)
+                    self.magic_stdout = self.open_text_file(file)
 
             except OSError as e:
                 if self.debug:
@@ -469,7 +571,7 @@ class JcCli():
 
                 error_msg = os.strerror(e.errno)
                 utils.error_message([
-                    f'"{self.magic_run_command_str}" file could not be opened: {error_msg}.'
+                    f'"{file}" file could not be opened: {error_msg}.'
                 ])
                 self.exit_error()
 
@@ -478,7 +580,7 @@ class JcCli():
                     raise
 
                 utils.error_message([
-                    f'"{self.magic_run_command_str}" file could not be opened. For details use the -d or -dd option.'
+                    f'"{file}" file could not be opened. For details use the -d or -dd option.'
                 ])
                 self.exit_error()
 
@@ -513,7 +615,7 @@ class JcCli():
 
     def set_parser_module_and_parser_name(self) -> None:
         if self.magic_found_parser:
-            self.parser_module = _get_parser(self.magic_found_parser)
+            self.parser_module = get_parser(self.magic_found_parser)
             self.parser_name = self.parser_shortname(self.magic_found_parser)
 
         else:
@@ -522,7 +624,7 @@ class JcCli():
                 self.parser_name = self.parser_shortname(arg)
 
                 if self.parser_name in parsers:
-                    self.parser_module = _get_parser(arg)
+                    self.parser_module = get_parser(arg)
                     found = True
                     break
 
@@ -534,9 +636,142 @@ class JcCli():
             utils.error_message(['Missing piped data. Use "jc -h" for help.'])
             self.exit_error()
 
+    def add_metadata_to_output(self) -> None:
+        """
+        This function mutates self.data_out in place. If the _jc_meta field
+        does not already exist, it will be created with the metadata fields. If
+        the _jc_meta field already exists, the metadata fields will be added to
+        the existing object.
+
+        In the case of an empty list (no data), a dictionary with a _jc_meta
+        object will be added to the list. This way you always get metadata,
+        even if there are no results.
+        """
+        if self.run_timestamp:
+            meta_obj: JSONDictType = {
+                'parser': self.parser_name,
+                'timestamp': self.run_timestamp.timestamp(),
+                'slice_start': self.slice_start,
+                'slice_end': self.slice_end
+            }
+
+            if self.magic_run_command:
+                meta_obj['magic_command'] = self.magic_run_command
+                meta_obj['magic_command_exit'] = self.magic_returncode
+
+            if self.inputlist:
+                meta_obj['input_list'] = self.inputlist
+
+            if isinstance(self.data_out, dict):
+                if '_jc_meta' not in self.data_out:
+                    self.data_out['_jc_meta'] = {}
+
+                self.data_out['_jc_meta'].update(meta_obj)
+
+            elif isinstance(self.data_out, list):
+                if not self.data_out:
+                    self.data_out.append({})
+
+                for item in self.data_out:
+                    if isinstance(item, dict):
+                        if '_jc_meta' not in item:
+                            item['_jc_meta'] = {}
+
+                        item['_jc_meta'].update(meta_obj)
+
+            else:
+                utils.error_message(['Parser returned an unsupported object type.'])
+                self.exit_error()
+
+    def slicer(self) -> None:
+        """Slice input data lazily, if possible. Updates self.data_in"""
+        if self.slice_str:
+            slice_start_str, slice_end_str = self.slice_str.split(':', maxsplit=1)
+            if slice_start_str:
+                self.slice_start = int(slice_start_str)
+            if slice_end_str:
+                self.slice_end = int(slice_end_str)
+
+        self.data_in = utils.line_slice(self.data_in, self.slice_start, self.slice_end)
+
+    def create_slurp_output(self) -> None:
+        """
+        Slurp input into a list. If input is coming from multiple /proc files
+        using magic syntax, then also add a `_file` key to the output.
+
+        If --meta-out is used then further wrap the data in a dict like so:
+            {"result": data}
+
+        self.input_list will already exist if the data is coming from the
+        /proc magic sytnax. Otherwise this funcion will build it for normal
+        slurp items.
+
+        This will allow --meta-out to add its information in a clean way.
+
+        This method updates self.data_out
+        """
+        if self.parser_module and isinstance(self.data_in, (str, Iterable)):
+            self.data_out = []
+
+            # single-line string parsers
+            if isinstance(self.data_in, str):
+                items = self.data_in.splitlines()
+                self.inputlist = []
+
+                for line in items:
+                    line = line.strip()
+                    self.inputlist.append(line)
+                    parsed_line = self.parser_module.parse(
+                        line,
+                        raw=self.raw,
+                        quiet=self.quiet
+                    )
+
+                    self.data_out.append(parsed_line)
+
+            # multiple files from /proc magic syntax
+            elif isinstance(self.data_in, List) and self.inputlist:
+                items = self.data_in
+
+                for mline in zip(self.inputlist, items):
+                    parsed_line = self.parser_module.parse(
+                        mline[1],
+                        raw=self.raw,
+                        quiet=self.quiet
+                    )
+
+                    if isinstance(parsed_line, dict):
+                        parsed_line.update({'_file': mline[0]})
+
+                    elif isinstance(parsed_line, List):
+                        for obj in parsed_line:
+                            obj.update({'_file': mline[0]})
+
+                    self.data_out.append(parsed_line)
+
+            if self.meta_out:
+                self.data_out = {"result": self.data_out}
+                self.run_timestamp = datetime.now(timezone.utc)
+                self.add_metadata_to_output()
+
+    def create_normal_output(self) -> None:
+        """standard output - updates self.data_out"""
+        if self.parser_module:
+            self.data_out = self.parser_module.parse(
+                self.data_in,
+                raw=self.raw,
+                quiet=self.quiet
+            )
+
+            if self.meta_out:
+                self.run_timestamp = datetime.now(timezone.utc)
+                self.add_metadata_to_output()
+
     def streaming_parse_and_print(self) -> None:
         """only supports UTF-8 string data for now"""
         self.data_in = sys.stdin
+        self.slicer()
+
         if self.parser_module:
             result = self.parser_module.parse(
                 self.data_in,
@@ -564,16 +799,13 @@ class JcCli():
         except UnicodeDecodeError:
             pass
 
-        if self.parser_module:
-            self.data_out = self.parser_module.parse(
-                self.data_in,
-                raw=self.raw,
-                quiet=self.quiet
-            )
+        self.slicer()
 
-            if self.meta_out:
-                self.run_timestamp = datetime.now(timezone.utc)
-                self.add_metadata_to_output()
+        if self.parser_module:
+            if self.slurp:
+                self.create_slurp_output()
+            else:
+                self.create_normal_output()
 
             self.safe_print_out()
 
@@ -587,62 +819,7 @@ class JcCli():
         exit_code = min(exit_code, MAX_EXIT)
         sys.exit(exit_code)
 
-    def add_metadata_to_output(self) -> None:
-        """
-        This function mutates data_out in place. If the _jc_meta field
-        does not already exist, it will be created with the metadata fields. If
-        the _jc_meta field already exists, the metadata fields will be added to
-        the existing object.
-
-        In the case of an empty list (no data), a dictionary with a _jc_meta
-        object will be added to the list. This way you always get metadata,
-        even if there are no results.
-        """
-        if self.run_timestamp:
-            meta_obj: MetadataType = {
-                'parser': self.parser_name,
-                'timestamp': self.run_timestamp.timestamp()
-            }
-
-            if self.magic_run_command:
-                meta_obj['magic_command'] = self.magic_run_command
-                meta_obj['magic_command_exit'] = self.magic_returncode
-
-            if isinstance(self.data_out, dict):
-                if '_jc_meta' not in self.data_out:
-                    self.data_out['_jc_meta'] = {}  # type: ignore
-
-                self.data_out['_jc_meta'].update(meta_obj)  # type: ignore
-
-            elif isinstance(self.data_out, list):
-                if not self.data_out:
-                    self.data_out.append({})
-
-                for item in self.data_out:
-                    if isinstance(item, dict):
-                        if '_jc_meta' not in item:
-                            item['_jc_meta'] = {}
-
-                        item['_jc_meta'].update(meta_obj)  # type: ignore
-
-            else:
-                utils.error_message(['Parser returned an unsupported object type.'])
-                self.exit_error()
-
-    def ctrlc(self, signum, frame) -> None:
-        """Exit on SIGINT"""
-        self.exit_clean()
-
-    def run(self) -> None:
-        # break on ctrl-c keyboard interrupt
-        signal.signal(signal.SIGINT, self.ctrlc)
-
-        # break on pipe error. need try/except for windows compatibility
-        try:
-            signal.signal(signal.SIGPIPE, signal.SIG_DFL)
-        except AttributeError:
-            pass
-
+    def _run(self) -> None:
         # enable colors for Windows cmd.exe terminal
         if sys.platform.startswith('win32'):
             os.system('')
@@ -657,6 +834,9 @@ class JcCli():
         # find options if magic_parser did not find a command
         if not self.magic_found_parser:
             for opt in self.args:
+                if SLICER_RE.match(opt):
+                    self.slice_str = opt
+
                 if opt in long_options_map:
                     self.options.extend(long_options_map[opt][0])
 
@@ -669,10 +849,12 @@ class JcCli():
         self.force_color = 'C' in self.options
         self.help_me = 'h' in self.options
         self.show_hidden = self.options.count('h') > 1   # verbose help
+        self.show_categories = self.options.count('h') > 2
         self.pretty = 'p' in self.options
         self.quiet = 'q' in self.options
         self.ignore_exceptions = self.options.count('q') > 1
         self.raw = 'r' in self.options
+        self.slurp = 's' in self.options
         self.meta_out = 'M' in self.options
         self.unbuffer = 'u' in self.options
         self.version_info = 'v' in self.options
@@ -683,6 +865,9 @@ class JcCli():
         self.set_mono()
         self.set_custom_colors()
 
+        if self.quiet:
+            utils.CLI_QUIET = True
+
         if self.verbose_debug:
             tracebackplus.enable(context=11)  # type: ignore
 
@@ -715,6 +900,13 @@ class JcCli():
 
         # parse and print to stdout
         if self.parser_module:
+            if self.slurp and not _parser_is_slurpable(self.parser_module):
+                utils.error_message([
+                    f'Slurp option not available with the {self.parser_name} parser.',
+                    'Use "jc -hhh" to find compatible parsers.'
+                ])
+                self.exit_error()
+
             try:
                 if _parser_is_streaming(self.parser_module):
                     self.streaming_parse_and_print()
@@ -724,12 +916,16 @@ class JcCli():
                     self.standard_parse_and_print()
                     self.exit_clean()
 
+            except BrokenPipeError:
+                sys.stdout = None  # type: ignore
+
             except (ParseError, LibraryNotInstalled) as e:
                 if self.debug:
                     raise
 
                 utils.error_message([
-                    f'Parser issue with {self.parser_name}:', f'{e.__class__.__name__}: {e}',
+                    f'Parser issue with {self.parser_name}:',
+                    f'{e.__class__.__name__}: {e}',
                     'If this is the correct parser, try setting the locale to C (LC_ALL=C).',
                     f'For details use the -d or -dd option. Use "jc -h --{self.parser_name}" for help.'
                 ])
@@ -751,6 +947,24 @@ class JcCli():
                 ])
                 self.exit_error()
 
+    def run(self) -> None:
+        try:
+            self._run()
+
+        except KeyboardInterrupt:
+            utils.error_message(['Exit due to SIGINT.'])
+            self.exit_error()
+
+        except Exception as e:
+            if self.debug:
+                raise
+
+            utils.error_message([
+                'Exit due to unexpected error:',
+                f'{e.__class__.__name__}: {e}'
+            ])
+            self.exit_error()
+
 
 def main():
     JcCli().run()
