@@ -100,7 +100,7 @@ def error_message(message_lines: List[str]) -> None:
 
     Returns:
 
-        None - just prints output to STDERR
+        None - just prints output to `STDERR`
     """
     columns = shutil.get_terminal_size().columns
 
@@ -148,7 +148,7 @@ def compatibility(mod_name: str, compatible: List[str], quiet: bool = False) ->
                       the parser. compatible options:
                       linux, darwin, cygwin, win32, aix, freebsd
 
-        quiet:        (bool) suppress compatibility message if True
+        quiet:        (bool) suppress compatibility message if `True`
 
     Returns:
 
@@ -169,7 +169,7 @@ def has_data(data: Union[str, bytes]) -> bool:
     Checks if the string input contains data. If there are any
     non-whitespace characters then return `True`, else return `False`.
 
-    For bytes, returns True if there is any data.
+    For bytes, returns `True` if there is any data.
 
     Parameters:
 
@@ -177,9 +177,9 @@ def has_data(data: Union[str, bytes]) -> bool:
 
     Returns:
 
-        Boolean      True if input string (data) contains non-whitespace
-                     characters, otherwise False. For bytes data, returns
-                     True if there is any data, otherwise False.
+        Boolean      `True` if input string (data) contains non-whitespace
+                     characters, otherwise `False`. For bytes data, returns
+                     `True` if there is any data, otherwise `False`.
     """
     if isinstance(data, str):
         return bool(data and not data.isspace())
@@ -210,7 +210,7 @@ def remove_quotes(data: str) -> str:
 
 
 def normalize_key(data: str) -> str:
-    """
+    r"""
     Normalize a key name by shifting to lower-case and converting special
     characters to underscores.
 
@@ -229,7 +229,7 @@ def normalize_key(data: str) -> str:
 
         string
     """
-    special = '''!"#$%&'()*+,-./:;<=>?@[\]^`{|}~ '''
+    special = r'''!"#$%&'()*+,-./:;<=>?@[\]^`{|}~ '''
     initial_underscore = False
     data = data.strip().lower()
 
@@ -365,7 +365,11 @@ def convert_to_bool(value: object) -> bool:
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
+def convert_size_to_int(
+    size: str,
+    binary: bool = False,
+    posix_mode: bool = False,
+    decimal_bias: bool = False) -> Optional[int]:
     """
     Parse a human readable data size and return the number of bytes.
 
@@ -375,6 +379,12 @@ def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
         binary:         (boolean) `True` to use binary multiples of bytes
                         (base-2) for ambiguous unit symbols and names,
                         `False` to use decimal multiples of bytes (base-10).
+        posix_mode:     (boolean) Treat one-letter units (k, m, g, etc.) as
+                        binary.
+        decimal_bias:   (boolean) `True` to treat slightly ambiguous two-
+                        letter unit symbols ending in "i" (e.g. Ki, Gi) to
+                        use decimal multiples of bytes (base-10). `False`
+                        (default) to use binary multiples of bytes.
     Returns:
 
         integer/None    Integer if successful conversion, otherwise None
@@ -394,6 +404,10 @@ def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
         1000
         >>> convert_size_to_int('1 KiB')
         1024
+        >>> convert_size_to_int('1 Ki')
+        1024
+        >>> convert_size_to_int('1 Ki', decimal_bias=True)
+        1000
         >>> convert_size_to_int('1 KB', binary=True)
         1024
         >>> convert_size_to_int('1.5 GB')
@@ -401,6 +415,9 @@ def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
         >>> convert_size_to_int('1.5 GB', binary=True)
         1610612736
     """
+    # normalize input by removing commas
+    size = size.replace(',', '')
+
     def tokenize(text: str) -> List[str]:
         tokenized_input: List = []
         for token in re.split(r'(\d+(?:\.\d+)?)', text):
@@ -438,17 +455,33 @@ def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
             # Convert plural units to singular units, for details:
             # https://github.com/xolox/python-humanfriendly/issues/26
             normalized_unit = normalized_unit.rstrip('s')
+
+            # Handle POSIX mode units where `k`, `m`, etc. are treated as binary
+            # https://www.gnu.org/software/coreutils/manual/html_node/Block-size.html
+            if len(normalized_unit) == 1 and posix_mode:
+                normalized_unit = normalized_unit + 'ib'
+
+            # Handle two-letter units (Ki, Gi, etc.) These are somewhat
+            # ambiguous, but are treated as binary by default. This can be
+            # changed with the `decimal_bias` parameter
+            if len(normalized_unit) == 2 and normalized_unit[1].lower() == 'i':
+                if decimal_bias:
+                    normalized_unit = normalized_unit[0]
+                else:
+                    normalized_unit = normalized_unit + 'b'
+
             for unit in disk_size_units:
                 # First we check for unambiguous symbols (KiB, MiB, GiB, etc)
                 # and names (kibibyte, mebibyte, gibibyte, etc) because their
                 # handling is always the same.
                 if normalized_unit in (unit.binary.symbol.lower(), unit.binary.name.lower()):
                     return int(tokens[0] * unit.binary.divider)
+
                 # Now we will deal with ambiguous prefixes (K, M, G, etc),
                 # symbols (KB, MB, GB, etc) and names (kilobyte, megabyte,
                 # gigabyte, etc) according to the caller's preference.
-                if (normalized_unit in (unit.decimal.symbol.lower(), unit.decimal.name.lower()) or
-                        normalized_unit.startswith(unit.decimal.symbol[0].lower())):
+                if (normalized_unit in (unit.decimal.symbol.lower(), unit.decimal.name.lower())
+                    or normalized_unit.startswith(unit.decimal.symbol[0].lower())):
                     return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
     # We failed to parse the size specification.
     return None
@@ -663,12 +696,14 @@ class timestamp:
             {'id': 1700, 'format': '%m/%d/%Y, %I:%M:%S %p', 'locale': None},  # Windows english format wint non-UTC tz (found in systeminfo cli output): 3/22/2021, 1:15:51 PM (UTC-0600)
             {'id': 1705, 'format': '%m/%d/%Y, %I:%M:%S %p %Z', 'locale': None},  # Windows english format with UTC tz (found in systeminfo cli output): 3/22/2021, 1:15:51 PM (UTC)
             {'id': 1710, 'format': '%m/%d/%Y, %I:%M:%S %p UTC%z', 'locale': None},  # Windows english format with UTC tz (found in systeminfo cli output): 3/22/2021, 1:15:51 PM (UTC+0000)
+            {'id': 1720, 'format': '%A, %B %d, %Y %I:%M:%S %p', 'locale': None},  # ipconfig cli output format: Thursday, June 22, 2023 10:39:04 AM
             {'id': 1750, 'format': '%Y/%m/%d-%H:%M:%S.%f', 'locale': None},  # Google Big Table format with no timezone: 1970/01/01-01:00:00.000000
             {'id': 1755, 'format': '%Y/%m/%d-%H:%M:%S.%f%z', 'locale': None},  # Google Big Table format with timezone: 1970/01/01-01:00:00.000000+00:00
             {'id': 1760, 'format': '%Y-%m-%d %H:%M:%S%z', 'locale': None},  # certbot format with timezone: 2023-06-12 01:35:30+00:00
             {'id': 1800, 'format': '%d/%b/%Y:%H:%M:%S %z', 'locale': None},  # Common Log Format: 10/Oct/2000:13:55:36 -0700
             {'id': 2000, 'format': '%a %d %b %Y %I:%M:%S %p %Z', 'locale': None},  # en_US.UTF-8 local format (found in upower cli output): Tue 23 Mar 2021 04:12:11 PM UTC
             {'id': 3000, 'format': '%a %d %b %Y %I:%M:%S %p', 'locale': None},  # en_US.UTF-8 local format with non-UTC tz (found in upower cli output): Tue 23 Mar 2021 04:12:11 PM IST
+            {'id': 3100, 'format': '%a %d %b %Y %I:%M:%S %p %z', 'locale': None},  # pacman format - append 00 to end to make it work: # Sat 11 May 2024 06:14:19 AM +0800
             {'id': 3500, 'format': '%a, %d %b %Y %H:%M:%S %Z', 'locale': None},  # HTTP header time format (always GMT so assume UTC): Wed, 31 Jan 2024 00:39:28 GMT
             {'id': 4000, 'format': '%A %d %B %Y %I:%M:%S %p %Z', 'locale': None},  # European-style local format (found in upower cli output): Tuesday 01 October 2019 12:50:41 PM UTC
             {'id': 5000, 'format': '%A %d %B %Y %I:%M:%S %p', 'locale': None},  # European-style local format with non-UTC tz (found in upower cli output): Tuesday 01 October 2019 12:50:41 PM IST
